/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: EarManager.java,v 1.4 2003/09/19 15:07:47 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen;

//import java
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Vector;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.jar.JarEntry;
import java.util.jar.JarException;
import java.util.jar.JarFile;
import java.util.zip.ZipEntry;

//import xml
import org.objectweb.jonas_ws.wsgen.xml.ApplicationXml;

//import jonas_lib
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;

/**
 * Use xml classes to get and write
 * informations inside an ear file.
 */

public class EarManager extends JarManager {
    
    private static final String DEFAULT_EAR = "application.xml";

    private static final String EAR = "META-INF" + File.separator + DEFAULT_EAR;

    private ApplicationXml application = null;

    /**
     * Build a new instance of a EarManager.
     *
     * @param earFilename the name of the ear file
     * that we want to manage.
     *
     * @throws WsGenException if the ear file doesn't exist.
     */
    public EarManager (String earFilename) 
	throws WsGenException {
	super(earFilename);
	// get the ear file DD.
	if (!isNewFile()){
	    try {
		JarFile jarFile = new JarFile(this.jarFileName);
		InputStream is = jarFile.getInputStream(jarFile.getEntry(EAR));
		application = loadApplication(is, EAR);
		is.close();
	    }
	    catch (IOException ioe) {
		if (logger.isLoggable(BasicLevel.WARN))
		    logger.log(BasicLevel.WARN, "Cannot get '" + EAR +
			       "' JarEntry from '" + jarFileName + "'\n" +
			       "will continue with default application.xml");	
		application = createDefaultApplicationDesc();
	    }  
	}else{
	    System.out.println("It's a new file : create a default web desc");
	    application = createDefaultApplicationDesc();
	}

    }

    /**
     * Add a new War in the current ear file.
     *
     * @param warFile the new war file name.
     *
     * @throws WsGenException
     */
    public void addWar (String warFileName)
	throws WsGenException {

	try {
	    File war = new File(warFileName) ;
	    String warName = war.getName();
	    addFile(warName, war);
	    application.addWeb(warName, warName.substring(0, warName.length()-4));
	}
	catch (Exception e) {
	    throw new WsGenException(e);
	}
    }

    /**
     * Add a new EjbJar in the current ear file.
     *
     * @param ejbjarFile the new ejbjar file name.
     *
     * @throws WsGenException
     */
    public void addEjbjar (String ejbjarFileName)
	throws WsGenException {

	try {
	    File jar = new File(ejbjarFileName) ;
	    String ejbName = jar.getName();
	    addFile(ejbName, jar);
	    application.addEjb(ejbName);
	}
	catch (Exception e) {
	    throw new WsGenException(e);
	}
    }

        protected void save() throws WsGenException {
	
	if (!isNewFile()){
	    // remove the application dd entrie in temp jar entries
	    Vector validEntries = new Vector();		
	    for (Enumeration e = tjEntries; e.hasMoreElements();) {		    
		ZipEntry ze = (ZipEntry)(e.nextElement());
		if(!ze.getName().equals(EAR)){		    
		    validEntries.add(ze);
		}
	    }
	    tjEntries = validEntries.elements();
	}
	
	super.save();
	try {
	    jos.putNextEntry(new JarEntry(EAR));
	    application.marshal(jos);
	}
	catch (Exception e) {
	    throw new WsGenException("Error when saving files in '"+jarFileName+"'",e);
	}
    }


    private ApplicationXml createDefaultApplicationDesc() throws WsGenException{

	if (logger.isLoggable(BasicLevel.DEBUG))
	    logger.log(BasicLevel.DEBUG, "Create Default application.xml for '" + jarFileName + "'");

        String jonasRoot = System.getProperty("install.root");
        if (jonasRoot == null) {
            throw new WsGenException("System property 'install.root' not set");
        }
        String path2Tmpl = new String(jonasRoot + File.separatorChar + "templates"
                                      + File.separatorChar + "wsgen");
	InputStream is = null;
	String filename = path2Tmpl + File.separatorChar + DEFAULT_EAR;
	try {
	    is = new FileInputStream(filename);
	}
	catch (FileNotFoundException ignored) {}

	ApplicationXml app =  new ApplicationXml(is);
	app.removeModules();
	return app;
    }

    /** 
     * Load the application.xml file.
     * @param is the InputStream of the XML file.
     * @param fileName the name of the file (application.xml).
     * @return a structure containing the result of the application.xml parsing.
     * @throws WsGenException if the deployment descriptor 
     * is corrupted.
     */
    private static ApplicationXml loadApplication(InputStream is, String fileName) 
        throws WsGenException {
	return new ApplicationXml(is);
    }


}
