/*
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 1999 Bull S.A.
 * Contact: jonas-team@objectweb.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public 
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * Initial developer(s): Xavier Delplanque & Guillaume Sauthier
 * --------------------------------------------------------------------------
 * $Id: JarManager.java,v 1.4 2003/09/18 14:07:25 delplanx Exp $
 * --------------------------------------------------------------------------
*/

package org.objectweb.jonas_ws.wsgen;

import java.util.zip.ZipException;
import java.util.zip.ZipEntry;

import java.util.jar.JarEntry;
import java.util.jar.JarException;
import java.util.jar.JarOutputStream;
import java.util.jar.JarFile;
import java.util.jar.Manifest;

import java.util.Enumeration;
import java.util.Hashtable;

import java.io.IOException;
import java.io.FileOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.File;
import java.io.InputStream;

// SAX imports
import org.xml.sax.SAXParseException;

// import logger
import org.objectweb.jonas.common.Log;
import org.objectweb.util.monolog.api.Logger;
import org.objectweb.util.monolog.api.BasicLevel;


/**
 * Manage a jar file.
 */
public class JarManager {
    
    protected String jarFileName;
    protected JarOutputStream jos;
    protected File tempJar;
    protected Enumeration tjEntries;   
    protected Hashtable jarEntries = new Hashtable();

    private boolean newFile = false;

    /** The logger to use */
    protected static Logger logger = Log.getLogger(Log.JONAS_WSGEN_PREFIX);

    /**
     * Build a new instance of a JarManager.
     * Create the file if it doesn't exists.
     *
     * @param jarFilename the name of the jar file
     * that we want to manage.
     *
     * @throws WsGenException if the war file doesn't exist.
     */
    public JarManager (String jarFilename) 
	throws WsGenException {
	try {
      	    File jar = new File(jarFilename);
	    this.jarFileName = jar.getCanonicalPath();
	    	System.out.println("JAR EXISTS ? : "+jar.exists());
	    if (!jar.exists()) {
		newFile = true;
		createFile(jar);
	    }
	}
	catch (IOException ioe) {
	    throw new WsGenException("cannot find/open/create the jar file '" +
				     jarFileName + "'", ioe);
	}
    }
    
    /**
     * Add all Files contained in a directory in the current jar file.
     * This adding keep the same file structure (directory & subdir)
     *
     * @param dir the directory to add
     *
     * @throws WsGenException
     */
    public void addDir (File dir) 
	throws WsGenException{
	String base = "";
	addDir (base, dir);
    }

    /**
     * Add all Files contained in a directory in the current jar file 
     * at the specified entryName location.
     * This adding keep the same file structure (directory & subdir)
     *
     * @param base where to put directory files in the jar
     * @param dir the directory to add
     *
     * @throws WsGenException
     */
    public void addDir (String base, File dir)
	throws WsGenException{
	
	try {
	File[] files = dir.listFiles();
	// add each repertory file and explore subdirectories
	for (int i = 0; i < files.length; i++) {
	    File crtFile = files[i];	    
	    if (crtFile.isFile()) {
		// add the file in the jar
		addFile(base+crtFile.getName(), crtFile);
	    }
	    else {
		// explore the subdirectory
		addDir(base+crtFile.getName()+"/", crtFile);
	    }
	}
	} catch (Exception e) {
	    throw new WsGenException("Error when adding files for directory '"+dir.getName()+"'",e);
	}
    }

    /**
     * Add a new File in the current jar file.
     *
     * @param entryName where to put the file in the jar
     * @param file the file to add
     *
     * @throws WsGenException
     */
    public void addFile (String entryName, File file)
	throws WsGenException {	

	// get the zip entry for the given entry name
	String filename = file.getName();
	JarEntry je;
	try {
	    je = new JarEntry(entryName);
	}
	catch (NullPointerException npe) {
	    throw new WsGenException("The given file name '" + entryName + "' is null",npe);
	}
	catch (IllegalArgumentException iae) {
	    throw new WsGenException("The given file name '" + entryName + "' is too long",iae);
	}
	
	// add this entry in the entries
	try {
	    jarEntries.put(je, file);
	}
	catch (NullPointerException npe) {
	    throw new WsGenException("Null Jar entry added '" + entryName + "'", npe);
	}
	
    }

    /**
     * Create an empty file in the File System
     * @param file the file to create
     * @throws IOException if an IO error occurs
     */
    private void createFile(File file) throws IOException, WsGenException {
	
	if (logger.isLoggable(BasicLevel.DEBUG))
	    logger.log(BasicLevel.DEBUG, "Create File : '" + file.getPath() + "'");
	File parent = file.getCanonicalFile().getParentFile();
	// if parent dir not existant
	// create the directories
	if (!parent.exists()) {
	    parent.mkdirs();
	}

    }

    /**
     * build a message from SAX Exception in a consistent style, consistent
     * with emacs compile mode (same as grep, cc, javac, etc).
     * @param fileName name of the file
     * @param exception the SAX exception
     * @param msg the string message
     * @return a message which is in a consistent style
     */
    protected  static String getSAXMsg(String fileName,
                                       SAXParseException exception, String msg) {
        String ret = fileName + ":" + exception.getLineNumber() + ":" +
            exception.getColumnNumber() + ": ";
        ret += msg;
        return ret;
    }

    /**
     * Return true if a new jar file has been created.
     * @return true if a new jar file has been created.
     */
    protected boolean isNewFile () {
	return newFile;
    }

    /**
     * write a new entry in the jar output stream.
     * @param ze the zip entry to add.
     * @param is the input stream to write.
     */
    protected void writeEntry (ZipEntry ze, InputStream is) 
	throws WsGenException {	

	try {
	    jos.putNextEntry(ze);
	}
	catch (ZipException zipe) {
	    zipe.printStackTrace(System.out);
	    throw new WsGenException("Jar entry format error for '" + ze.getName() + "'", zipe);
	}
	catch (IOException ioe) {
	    ioe.printStackTrace(System.out);
	    throw new WsGenException("Error when adding jar entry '" + ze.getName() + "'", ioe);
	}
		
	byte[] buf = new byte[1024];
	int length;
	
	// write the file in the jar
	try {
	    while ((length = is.read(buf)) > 0)
		jos.write(buf, 0, length);	
	}
	catch (Exception e) {
	    throw new WsGenException("Error writing entry '" + ze.getName() + "'", e);
	}

    }

    /**
     * Open the output stream on the jar file and create a save of the jar.
     */
    protected void open () 
	throws WsGenException {
	
	if (!isNewFile()){
	    // create a temporary jar file
	    System.out.println("create a temporary jar file");
	    FileOutputStream os;
	    FileInputStream is;
	    try {
		tempJar = File.createTempFile("wsgen", ".jar") ;
		os = new FileOutputStream(tempJar);
		is = new FileInputStream(jarFileName);
	    }
	    catch (Exception e) {
		e.printStackTrace(System.out);
		throw new WsGenException("Error creating jar copy '"+tempJar.getName()+"'", e);
	    }
	    
	    // copy the file
	    System.out.println("copy the file");
	    byte[] buf = new byte[1024];
	    int length;
	    try {
		while ((length = is.read(buf)) > 0)
		    os.write(buf, 0, length);	
		is.close();
		os.flush();
		os.close();

		// record jar entries
		JarFile tj = new JarFile(tempJar);
		tjEntries = tj.entries();
	    }
	    catch (Exception e) {
		throw new WsGenException("Error writing file '"+tempJar.getName()+"'", e);
	    }	
	    
	}
   
	// open an output stream on the jar file
	System.out.println("open an output stream on the jar file");
	try {	    
	    jos = new JarOutputStream(new FileOutputStream(jarFileName));
	} catch (IOException ioe) {
	    throw new WsGenException("cannot open output stream on the jar file '" +
				     jarFileName + "'", ioe);
	}
    }

    /**
     * add all jar entries in the jar file.
     */    
    protected void save ()
	throws WsGenException {	
	System.out.println ("add temp jar entries");
	ZipEntry ze;

	System.out.println ("add file entries");
	// add file entries
	File file = null;
	try {
	    for (Enumeration e = jarEntries.keys(); e.hasMoreElements();) {
		ze = (ZipEntry)(e.nextElement());
		file = (File)(jarEntries.get(ze));
		System.out.println ("file"+file);
		writeEntry(ze, new FileInputStream(file));
	    }	    
	}
	catch (Exception e) {
	    throw new WsGenException("Error when adding file '" + file.getName() + "' in jar", e);
	}

	if(!isNewFile()) {
	    // add temp jar entries
	    System.out.println ("add temp entries");
	    try {
		JarFile tj = new JarFile(tempJar);
		for (Enumeration e = tjEntries; e.hasMoreElements();) {		    
		    ze = (ZipEntry)(e.nextElement());
		    if (!jarEntries.containsKey(ze)){
			System.out.println ("add entry "+ze.getName());
			writeEntry(ze, tj.getInputStream(ze));
		    }
		}
	    }
	    catch (IOException ioe) {
		ioe.printStackTrace(System.out);
		throw new WsGenException("Error when adding temporary jar '" + tempJar.getPath() + "' entries", ioe);
	    }
	}
    }

    public void close ()
	throws WsGenException {	
	System.out.println ("open");
	open();
	System.out.println ("save");
	save();
	System.out.println ("close");
	try {
	    jos.flush();
	    jos.close();
	}
	catch (Exception e){
	    if (logger.isLoggable(BasicLevel.DEBUG))
		logger.log(BasicLevel.ERROR, "error when closing output stream");
	}
	if (!isNewFile() && !tempJar.delete())
	    System.out.println("Error when delete temp file '"+tempJar.getName()+"'");
    }
        
}
